/* LinuxBot Client - control, via TCP/IP, of LinuxBot from remote client

   Copyright (c) 2000 University of the West of England, Bristol
   
   Intelligent Autonomous Systems Lab
   Faculty of Engineering
   UWE, Bristol
   
   Alan.Winfield@uwe.ac.uk
   
   TO COMPILE:
       gcc lclient.c -o lclient -lncurses
       
   TO USE:
       1. ensure 'lserver' is running on a robot
       2. enter 'lclient xx.xx.xx.xx' with the robot's ip address
       3. assuming client and server programs connect ok,
          press up-arrow or 'f' to go forward
              down-arrow or 'b' to reverse
              left-arrow or 'l' to turn left
             right-arrow or 'r' to turn right
           the space bar or 's' to stop
          'x' to execute remote program, and quit
          'q' to halt, close the server, and quit.
*/

/*-----------------------------------------------------------------------------
   REVISION HISTORY
   
V1.1	21/9/00		Modified to allow execution of remote program
-----------------------------------------------------------------------------*/

/* Includes start here.. */
#include <curses.h>		/* for curses i/o */

#include <sys/types.h>		/* sockets API includes... */
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>


/* Defines start here.. */
#define SERVPORT 1025		/* Port no for the tcp/ip connection */
#define BUFLEN   128		/* length char buffers */

#define FWD_SPEED 50		/* initial forward speed */
#define RVS_SPEED 40		/* initial reverse speed */
#define TURN_SPEED 30		/* initial turn speed */
#define INCR_SPEED 5		/* speed change increment */


/* echo the motor speed values l and r to line 4 of the terminal */
void echomotors( int l, int r )
{
    	char buf[BUFLEN];
        sprintf(buf, "lmotor = %4d, rmotor = %4d ", l, r);
    	mvaddstr( 4, 0, buf );
    	refresh();
} 


/* send the motors 'on' command, with speed values l and r to the 
   connected socket */
void sendmotors( int sock, int l, int r )
{
	char buf[BUFLEN];
	
	int buflen = sprintf(buf, "6 %4d %4d ", l, r);
	write(sock, buf, buflen);
}


/* Hard limit the motor speed values, with addresses *l and *r */
void governor( int *l, int *r )
{
	if (*l > 100) *l = 100;
	if (*r > 100) *r = 100;
	if (*l < -100) *l = -100;
	if (*r < -100) *r = -100;
}


/* Main program start here.. */
int main (int argc, char **argv)
{
	int                ConnSock;	/* connecting socket */
	struct sockaddr_in ServAddr;	/* socket address structure */
	
    	int  done = FALSE;		/* flag for main while loop */
    	int  stopped = TRUE;		/* TRUE when robot is halted */
    	int  lmotor = 0, rmotor = 0;	/* current left and right motor speeds */
	char buf[BUFLEN];

	/* First check we have an argument.. */
	if (argc != 2)
	{
		printf("usage: lclient <ipaddress>\n");
		exit(1);
	}	
			
	/* Get a socket.. */
	ConnSock = socket(AF_INET, SOCK_STREAM, 0);
	if (ConnSock < 0)
	{
		perror("socket");
		exit(2);
	}
		
	/* Initialise local sockaddr_in structure.. */
	bzero(&ServAddr, sizeof(ServAddr));
	ServAddr.sin_family      = AF_INET;		/* address family */
	ServAddr.sin_port        = htons(SERVPORT);	/* port no */
/*	if (inet_pton(AF_INET, argv[1], &ServAddr.sin_addr) <= 0)
	{
		perror("inet_pton");
		exit(3);
	}  */
	if ( (ServAddr.sin_addr.s_addr = inet_addr( argv[1] )) == -1 )
	{
		perror("inet_addr");
		exit(3);
	}
	

	/* Attempt to connect.. */
	if (connect(ConnSock, (struct sockaddr *)&ServAddr, sizeof(ServAddr)) < 0)
	{
		perror("connect failed");
		exit(4);
	}
	
	(void) initscr();      /* initialize the curses library */
    	keypad(stdscr, TRUE);  /* enable keyboard mapping */
	(void) nonl();         /* tell curses not to do NL->CR/NL on output */
	(void) cbreak();       /* take input chars one at a time, no wait for \n */
	(void) noecho();       /* don't echo input */  
 
	sprintf(buf, "Connected to %s...", argv[1]);
	mvaddstr( 0, 0, buf );
	sprintf(buf, "Use arrow keys to drive, space bar to stop, 'q' to quit...");
	mvaddstr( 2, 0, buf );
	refresh();
	echomotors( 0, 0 );		/* start showing all stopped */
 
	while (!done) {

		int c = getch();	/* get a single character from the keyboard */

		switch(c) {

			/* drive the robot FORWARD... */
			case KEY_UP:
			case 'f':
				if (stopped)
					lmotor = rmotor = FWD_SPEED;
				else
				{   /* already going fwd, so increase speed */
					lmotor+=INCR_SPEED;
					rmotor+=INCR_SPEED;
					governor( &lmotor, &rmotor );
				}
				echomotors( lmotor, rmotor );
				sendmotors( ConnSock, lmotor, rmotor );
				stopped = FALSE;
				break;
	    
			/* drive the robot to REVERSE... */
			case KEY_DOWN:
			case 'b':
				if (stopped)
					lmotor = rmotor = -RVS_SPEED;
				else
				{
					lmotor -= INCR_SPEED;
					rmotor -= INCR_SPEED;
					governor( &lmotor, &rmotor );
				}
				echomotors( lmotor, rmotor );
				sendmotors( ConnSock, lmotor, rmotor );
				stopped = FALSE;
				break;
	    
			/* turn the robot to the RIGHT... */
			case KEY_RIGHT:
			case 'r':
				if (stopped)
				{
					lmotor = TURN_SPEED;
					rmotor = -TURN_SPEED;
				}
				else
				{
					lmotor += INCR_SPEED;
					rmotor -= INCR_SPEED;
					governor( &lmotor, &rmotor );
				}
				echomotors( lmotor, rmotor );
				sendmotors( ConnSock, lmotor, rmotor );
				stopped = FALSE;
				break;

			/* turn the robot to the LEFT... */
			case KEY_LEFT:
			case 'l':
				if (stopped)
				{
					lmotor = -TURN_SPEED;
					rmotor = TURN_SPEED;
				}
				else
				{
					lmotor -= INCR_SPEED;
					rmotor += INCR_SPEED;
					governor( &lmotor, &rmotor );
				}
				echomotors( lmotor, rmotor );
				sendmotors( ConnSock, lmotor, rmotor );
				stopped = FALSE;
				break;
	    
			/* all STOP... */
			case ' ':
			case 's':
				echomotors( 0, 0);
				write( ConnSock, "3 ", 2);
				stopped = TRUE;
				break;

			/* QUIT... */
			case 'q':
				echomotors( 0, 0);
				write( ConnSock, "3 ", 2);	/* stop motors */
				write( ConnSock, "99", 2);	/* then shut down server */
				done = TRUE;
				break;

			/* eXecute program... */
			case 'x':
				echomotors( 0, 0);
				write( ConnSock, "3 ", 2 );
				write( ConnSock, "199", 3 );
				done = TRUE;
				break;
				
			default:
	     			beep();
	    
		}   /* switch */
	}	/* while */
    
	close(ConnSock);	/* close socket */
	endwin();  		/* terminate curses */
}
